\name{CCTable11.1a}
\alias{CCTable11.1a}
\title{Cochran and Cox design}
\usage{
	data(CCTable11.1a)
}

\description{
	A blocked experimental design from Cochran and Cox (1957).

	A Partially balanced incomplete block in 3 reps, and 27 blocks.
}

\source{
	Cochran, W.G. and Cox, G.M. (1957). {\emph Experimental designs}. Wiley, N.Y.
}

\keyword{datasets}


\eof
\name{efficient.rounding}
\alias{efficient.rounding}
\title{Efficient Rounding}
\description{
 A vector of proportions is efficiently rounded to integers.
}
\usage{
efficient.rounding(proportions,n,random=TRUE)
}

\arguments{
	\item{proportions}{A vector of proportions.}
	\item{n}{The sum of the resulting integers.}
	\item{random}{If TRUE, ties will be broken at random, otherwise the first of the tied values will be used.}
}
\details{
	This function implements an efficient rounding procedure to round approximate theory designs
	into replicated integer approximations.
}
\value{
	A vector of replications summing to n.
}

\author{
	Bob Wheeler \email{bwheeler@echip.com}
	
	Please cite this program as follows:

Wheeler, R.E. (2004). efficient.rounding. \emph{AlgDesign}. The R project for statistical computing \url{http://www.r-project.org/}
}


\references{
  Pulkesheim, F. and Rieder, S. (1992). Efficient rounding of approximate designs. Biometrika. 79-4. 763-770.}
\keyword{design}

\eof
\name{eval.blockdesign}
\alias{eval.blockdesign}
\title{Evaluates a blocked design.}
\description{
 A blocked design is evaluated.
}
\usage{
eval.blockdesign(frml,design,blocksizes,rho=1,confounding=FALSE,center=FALSE)
}
\arguments{
	\item{frml}{The formula used to create the blocked design.}
	\item{design}{The blocked design, which may be the design output by optBlock().}
	\item{blocksizes}{A vector of blocksizes for the design.}
	\item{rho}{A vector, giving the ratios of whole to within variance components.}
	\item{confounding}{If confounding=TRUE, the confounding matrix will be output.} 
	\item{center}{If TRUE, numeric variables will be centered before frml is applied.}

}
\value{
	\item{confounding}{A matrix based on the design matrix in which the within block variables
		have been centered about their block means. The columns of this matrix which give 
		the regression coefficients of each variable regressed on the others. If \eqn{C} is the 
		confounding matrix, then \eqn{-XC} is a matrix of residuals of the variables regressed on 
		the other variables.}
	\item{determinant.all.terms.within.terms.centered}{\eqn{|M|^{1/k}}{(det(M)^(1/k)}, where 
		\eqn{M=X^TX/N}{M=X'X/N} and X is the model expanded \eqn{N\times k}{N x k} design matrix 
		in which the within block variables	have been centered about the grand mean.}
	\item{within.block.efficiencies}{The determinant criterion blocking efficiencies for the range 
		of rho's input. A high efficiency indicates that there is little intrablock information
		to be recovered in the analysis.}
	\item{block.centered.properties}{A matrix with four rows. The columns correspond to constant, whole
		block terms and within block terms:

		\enumerate{
			\item The degrees of freedom for terms in the expanded model.  
			\item The determinant of the block centered within block terms.  
			\item The geometric mean of the block centered variances.  
			\item The geometric mean of the ratio of centered to block centered variances. 
			}
		}
}

\author{
	Bob Wheeler \email{bwheeler@echip.com}
	
	Please cite this program as follows:

Wheeler, R.E. (2004). eval.blockdesign. \emph{AlgDesign}. The R project for statistical computing \url{http://www.r-project.org/}
}

\keyword{design}


\eof
\name{eval.design}
\alias{eval.design}
\title{Evaluates a design.}
\description{
 A design is evaluated.
}
\usage{
eval.design(frml,design,confounding=FALSE,variances=TRUE,center=FALSE,X=NULL)
}
\arguments{
	\item{frml}{The formula used to create the design.}
	\item{design}{The design, which may be the design part of the output of optFederov().}
	\item{confounding}{If confounding=TRUE, the confounding patterns will be shown.} 
	\item{variances}{If TRUE, the variances each term will be output.}
	\item{center}{If TRUE, numeric variables will be centered before frml is applied.}
	\item{X}{X is either the matrix describing the prediction space for I or for G, the 
		the candidate set from which the design was chosen. They are often
		the same.}
}
\note{
	I, Ge and Dea are calculated only when X is input.
}
\value{
	\item{confounding}{A matrix. The columns of which give the regression coefficients of
		each variable regressed on the others. If \eqn{C} is the confounding matrix, then 
		\eqn{-ZC} is a matrix of residuals of the variables regressed on the other variables.}
	\item{determinant}{\eqn{|M|^{1/k}}{(det(M/N)^(1/k)}, where \eqn{M=Z'Z/N}{M=Z'Z/N}, and Z is 
		the model expanded \eqn{N\times k}{N x k} design matrix.}
  	\item{A}{The average coefficient variance: \eqn{trace(Mi)/k}, where \eqn{Mi} is the 
		inverse of \eqn{M}.}
  	\item{I}{The average prediction variance over X, which can be shown to be 
		\eqn{trace((X'X*Mi)/N)}{trace((X'X*Mi)/N}.} 
  	\item{Ge}{The minimax normalized variance over X, expressed as an efficiency with respect 
		to the optimal approximate theory design. It is defined as \eqn{k/max(d)}, where 
		\eqn{max(d)} is the maximum normalized variance over \eqn{X} -- i.e. the max of 
		\eqn{x'(Mi)x}, over all rows \eqn{x'} of \eqn{X}.}
  	\item{Dea}{A lower bound on \code{D} efficiency for approximate theory designs. It is 
		equal to \eqn{exp(1-1/Ge)}.}
	\item{diagonality}{The diagonality of the design, excluding the constant, if any. Diagonality
		is defined as \eqn{(|M_1|/\prod{diag(M_1)})^{1/k}}{(|M1|/prod(diag(M1)))^(1/k)}, where 
		\eqn{M_1}{M1} is \eqn{M} with first column and row deleted when there is a constant.}
	\item{gmean.variances}{The geometric mean of the coefficient variances.}

}

\author{
	Bob Wheeler \email{bwheeler@echip.com}
	
	Please cite this program as follows:

Wheeler, R.E. (2004). eval.design. \emph{AlgDesign}. The R project for statistical computing \url{http://www.r-project.org/}
}

\keyword{design}

\eof
\name{expand.formula}
\alias{expand.formula}
\title{Expanding a formula}
\description{
 Formulas are expanded to accommodate special functions for continuous and mixture variables.
}
\usage{
expand.formula(frml,varNames,const=TRUE,numerics=NULL)
}

\arguments{
	\item{frml}{A formula starting with ~ in the usual way.}
	\item{varNames}{A list of variable names to be used when a dot is used as shorthand for ``all variables.''}
	\item{const}{If FALSE, the constant will be suppressed.}
	\item{numerics}{A vector the same length as varNames, with TRUE for corresponding numeric variables. If
		missing, all variables will be assumed to be numeric.}
}
\details{
This function expands formulas to accommodate polynomial models for which R has minimal support.
Assuming for illustration that there are three variables, A, B, and C, the following expressions
may be used. In addition, a dot may be used to indicate that all variables in varNames are to be used. 

All agruments to quad(), cubic(), and cubicS() must be numeric.

\itemize{
	\item \eqn{~.} makes \eqn{~ A + B + C}
	\item \eqn{~.^p} makes \eqn{~ (A + B + C)^p}, where p is an integer
	\item quad(A,B,C) makes \eqn{~(A+B+C)^2+I(A^2)+I(B^2)+I(C^2)}
	\item cubic(A,B,C) makes \eqn{~(A+B+C)^3+I(A^2)+I(B^2)+I(C^2)+I(A^3)+I(B^3)+I(C^3)}
 	\item cubicS(A,B,C) makes \eqn{~(A+B+C)^3+I(A*B*(A-B))+I(A*C*(A-C))+I(B*C*(B-C))}
}

The cubicS() function produces a non-singular representation of a cubic model, when the 
variables are mixture variables, that is when the rows of \code{data} sum to a constant
value, usually 1.0. Because of the mixture constraint, models containing mixture variables
should not have a constant term. The linear and quadratic models for mixture variables
A, B, and C are given by \eqn{-1+(A+B+C)} and \eqn{-1+(A+B+C)^2} respectively. See Gorman and Hinman [1962] for 
details.

}
\note{
	expand.formula() is called by model.matrix() through the method call model.matix.formula(), thus one may use the above special functions with model.matrix().
}

\value{
	An expanded formula is returned.
}
\author{
	Bob Wheeler \email{bwheeler@echip.com}
	
	Please cite this program as follows:

Wheeler, R.E. (2004). expand.formula. \emph{AlgDesign}. The R project for statistical computing \url{http://www.r-project.org/}
}

\references{
  Gorman, J.W. and Hinman, J.E. (1962). Simplex lattice designs for 
  multicomponent systems. \emph{Technometrics}. 4-4. 463-487.
}
\keyword{design}


\eof
\name{optFederov}
\alias{optFederov}
\title{Optimal design}

\usage{
optFederov(frml,data,nTrials,center=FALSE,approximate=FALSE,criterion="D",
	evaluateI=FALSE,space=NULL,augment=FALSE,rows,nullify=0,
	maxIteration=100,nRepeats=5,DFrac=1,CFrac=1,args=FALSE) 

}
\arguments{

	\item{frml}{This may be omitted if \code{data} is the fully model expanded candidate list.
	 If present it should be a formula starting with ~ which describes the model using
	 variable names from \code{data}. It may be ~. if all variables from \code{data} are to be used linearly.
       In addition to the usual operators, quad(), cubic() and cubicS() may be used to 
       expand variables from \code{data} into polynomial models.}
	\item{data}{The candidate list. A matrix or data.frame describing the variables. If a matrix is input and 
	 the columns are not named, they will be assigned names X1,X2, etc. If a data.frame is input without 
       column names, they will be named Var1, Var2, etc. Although data may be input as global variables used 
       in frml, it is preferable to input it here.}
	\item{nTrials}{If approximate=FALSE, it is the number of trials in the final design and if missing, it will be 
	 taken as the greater of \code{length(rows)} or 5 plus the number of terms in the model. If approximate=TRUE,
	 nTrials will be used to round the optimal proportions so that the replications of the points add to nTrials.}
	\item{approximate}{When FALSE, an exact design in nTrails will be calculated. When TRUE the proportions for
	 an approximate theory design will be calculated. If nTrials is set, any proportion less than 1/(2*maxIteration)
	 will be discarded before the proportions are efficiently rounded, otherwise all non-zero proportions will be 	 	 
	 shown: these are the support points.}
	\item{center}{When TRUE, the numeric variables will be centered.}
	\item{criterion}{"D", "A", or "I"}
	\item{evaluateI}{TRUE to evaluate and report I in addition to
	  other criteria. This parameter is included, because evaluating
	  I requires extra effort.}
	\item{space}{If the criterion is "I" or evaluate I is true, the space over which the I criterion is to
	  be evaluated may be input. It should be a matrix with the same column types and names as in data. If
	  space is not input the evaluation will be done over the space described by data.}
	\item{augment}{If TRUE, the row numbers in \code{rows} will never be exchanged.}
	\item{rows}{Either a vector of row numbers (not row names) from \code{data} to be used as the
	  starting design or a vector of row numbers for the design to be augmented. Note, replicate
	  row numbers will be discarded and the length of rows cannot exceed the number of rows in data.}
	\item{nullify}{When non-zero, the initial design is obtained by nullification. If \code{nullify=1},
 	 \code{nTrials} will be calculated (In this case nRepeats is set to 1). If \code{nullify=2}, 
	  number-of-terms trials will be calculated, and the remainder, up to \code{nTrials}, will be filled 
	out at random.}
	\item{maxIteration}{maximum number of times points are exchanged, within each repeat, in seeking an optimum design.}
	\item{nRepeats}{Number of times the entire process is repeated. Has no effect when approximate=TRUE, 
	 or when nullify=1.}
	\item{DFrac}{Design fraction: the fraction of design used in search: 1 uses all of
	  them, 0 uses only the one with the smallest variance.}
	\item{CFrac}{Candidate fraction: the fraction of candidate set searched : 1 uses all of
	  them, 0 uses only the one with the largest variance.}
	\item{args}{If TRUE, the actual arguments to the function including the starting random number
		 seed will be output.}
      }

\description{Calculates an exact or approximate algorithmic design for one of three criteria, using Federov's exchange algorithm.
      }
      
\details{
Let \eqn{E(y)=Zb}, where \eqn{y} is a vector of n observations, \eqn{Z} is an \eqn{n\times k}{n x k} matrix, and \eqn{b} is a vector of k parameters. The ``exact'' design problem is to find a matrix \eqn{Z}, with rows selected from a \eqn{N \times k}{N x k} matrix \eqn{X}, that is ``best'' in some sense. The matrix \eqn{X} can be a discretization of a continuous space or it can represent categories. In either case, the algorithmic design calculation is with respect to \eqn{X}, and not to some larger space containing the points.

Approximate designs weight the candidate points with a probability measure, which for practical purposes amounts to allowing unequal replication. 
  
The Federov(1972) algorithm starts with \eqn{n} points chosen from \eqn{X}. They may be chosen randomly or by nullification, a procedure which iteratively adds points from the null space of \eqn{X}, until a non-singular \eqn{n} point design is found. The Federov algorithm exchanges points in the \eqn{n} point design \eqn{Z} with points in \eqn{X-Z}, i.e. points not in \eqn{Z}, in order to optimize a criterion, and quits when no profitable exchanges are possible, or the input parameter \code{maxIteration} is reached. The quality of the result depends on the starting design and the result may represent a local optimum. The procedure is repeated \code{nRepeats} times in order to come nearer to a global optimum. The parameters \code{DFrac} and \code{CFrac} control the portions of \eqn{Z} and \eqn{X-Z} that are used.


The goal of algorithmic design is to maximize the information about the parameters. The information matrix is a matrix proportional to \eqn{M=Z'Z/n}, and various functions of \eqn{M} are chosen for optimization. The most popular of these is the \code{D} criterion, \eqn{|M|^{1/k}}{|M|^(1/k)}, which is thus a scaling of the ``generalized variance.'' Other criteria of interest involve the variance of predicted values, such as the \code{G} criterion, which is the minimax value of \eqn{d(x)=x'(Mi)x}, over \eqn{X}, where \eqn{Mi} is the inverse of \eqn{M}, and \eqn{x'} is a row of \eqn{X}; and the \code{I} criterion, which is the average value of \eqn{d(x)} in the experimental region. These criteria are invariant under linear transformations of the parameter vector, which frees them from a dependency on units of scale. Other criteria are not invariant, such as the largest eigenvalue of \eqn{M} or the \code{A} criterion, which is \eqn{trace(Mi)/k}: it is of course proportional to the average variance of the parameter estimates. The criteria \code{D}, \code{A}, and \code{I} are supported by \code{optFederov()}, and \code{G}, which is intimately connected to \code{D}, is reported.

The theoretical optimum value of \code{G} is known for approximate theory designs, and so \eqn{G_e}{Ge}, the \code{G} efficiency of \code{G} is available as a standard of design quality. It is especially useful, because \eqn{G_e}{Ge} provides a lower bound on \eqn{D_e}{De}, the \code{D} efficiency for approximate theory, to wit: \deqn{D_e\ge exp(1-1/G_e)}{De >= exp(1-1/Ge)}.

}

\note{
Algorithmic design is often used with continuous and mixture variables for which R has minimal support, thus the functions quad(), cubic(), and cubicS() may be used in frml. The translation is done with \link{expand.formula}.

Mixture variables are variables such that the rows of \code{data} sum to a constant value, usually unity. Because of the mixture constraint, models containing mixture variables should not have a constant term. The linear and quadratic models for mixture variables A, B, and C are given by \eqn{-1+(A+B+C)} and \eqn{-1+(A+B+C)^2} respectively. See Gorman and Hinman [1962] for 
details.

The function gen.mixture() generates a list of candidate points whose rows sum to unity.

Details of the calculations and expanded examples may be found in ``Comments on algorithmic design'' in the
doc subfolder of this project.
}

\references{
  Atkinson, A.C. and Donev, A.N. (1992). \emph{Optimum experimental
    designs}. Clarendon Press, Oxford.

  Gorman, J.W. and Hinman, J.E. (1962). Simplex lattice designs for 
   multicomponent systems. \emph{Technometrics}. 4-4. 463-487.

  Federov, V.V. (1972). \emph{Theory of optimal experiments}. Academic
   Press, N.Y.
  

}

\value{

  \item{D}{The kth root of the generalized variance: \eqn{det(M)^{1/k}}{det(M)^(1/k)}, where \eqn{det(M)} is the
    determinant of the normalized dispersion matrix \eqn{M} --
    i.e. \eqn{M=Z'Z/n}, where \eqn{Z=X[rows,]}}
  \item{A}{The average coefficient variance: \eqn{trace(Mi)/k}, where \eqn{Mi} is the inverse of \eqn{M}.}
  \item{I}{The average prediction variance over X, which can be shown to be \eqn{trace((X'X*Mi)/N)}, where
	N is the number of rows in X. This is calculated only when I is the criterion or when \code{evaluateI} is TRUE.}
  \item{Ge}{The minimax normalized variance over X, expressed as an efficiency with respect to the optimal approximate
	theory design. It is defined as \eqn{k/max(d)}, where \eqn{max(d)} is the maximum normalized
    	variance over \eqn{X} -- i.e. the max of \eqn{x'(Mi)x}, over all rows \eqn{x'}
    	of \eqn{X}.}
  \item{Dea}{A lower bound on \code{D} efficiency for approximate theory designs. It is equal to \eqn{exp(1-1/Ge)}.}
  \item{design}{The design.} 
  \item{rows}{A numerical vector of the design row numbers.}
  \item{args}{A list of the actual arguments used in this call.}
}


\examples{

# EXAMPLE 1
# A quadratic polynomial in three variables. The resulting D will be about 0.46.
# This may be compared with a standard central composite design obtained from
# rows 1,3,5,7,9,11,13,15,17,19,21,23,25,27 of dat, which has a D value of 0.46.
# The central composite design seems to be the optimal design for all three criteria.

dat<-gen.factorial(levels=3,nVars=3,varNames=c("A","B","C"))

desD<-optFederov(~quad(A,B,C),dat,nTrials=14,eval=TRUE)
desA<-optFederov(~quad(.),dat,nTrials=14,eval=TRUE,crit="A")
desI<-optFederov(~quad(.),dat,nTrials=14,eval=TRUE,crit="I")

rows<-c(1,3,5,7,9,11,13,15,17,19,21,23,25,27)
desO<-optFederov(~quad(.),dat,nTrials=14,eval=TRUE,rows=rows)

# The I criterion may be seen to decrease as the space is expanded. 

levels<-seq(-1,1,by=.1)
dat<-expand.grid(list(A=levels,B=levels,C=levels))

desL<-optFederov(~quad(.),dat,nTrials=14,eval=TRUE)

# This is not the case for A or D. For A and D, the support points are the points 
# of the grid with the three levels above. Points not on this grid move
# the criteria in a non-optimal direction; hence, the enlarging space has no effect.

# EXAMPLES 2
# Standard designs are usually optimal designs. If nTrials is set to that for
# a standard design, and if nRepeats is large enough, the standard design will 
# often be found For example, a half replicate of a 2^4 will be obtained by the 
# following. 

dat<-gen.factorial(levels=2,nVars=3,varNames=c("A","B","C"))
desH<-optFederov(~.,dat,8)

# A third replicate of a 3^3 will be obtained by the following:

dat<-gen.factorial(levels=3,nVars=3,factor=1:3)
desT<-optFederov(~.,dat,9)

# An orthogonal design similar to a 12 run Plackett-Burman design can be 
# created by the following. 

dat<-gen.factorial(levels=2,nVars=11,varNames=c("A","B","C","D","E","F","G","H","J","K","L"))
desPB<-optFederov(~.,dat,12,nRepeats=20)

# The above calculation is numerically difficult for the A and I criteria, 
# and nRepeats=100 or more may be needed. 

# It is instructive to examine a case in which the standard design is not found.
# The following is an attempt to create a Latin square design. It is not always successful.

lv<-factor(1:5)
dat<-expand.grid(A=lv,B=lv,C=lv)
desL<-optFederov(~.,dat,nTrials=25,nRep=100)

# It may be summarized as follows.

cs<-xtabs(~.,desL$design)
{xx<-matrix(0,5,5); for (i in 1:5) xx=xx+cs[1:5,1:5,i]*i;xx}
 


# EXAMPLE 3
# Mixture variables have a constant sum, usually 1. This causes a linear dependency
# among terms of polynomial models. In particular the constant term is dependent.
# Squared terms in a quadratic model are confounded with interaction terms, so that
# a quadratic model for three mixture variables is ~0+(A+B+C)^2. The following
# calculation generates a set of candidate varibles using gen.mixture() with
# four values on each axis, and then creates a 15 run design. The design is optimal.
# Indeed, the candidate set produced by gen.mixture(2,5) is optimal. Note: 
# nullify=TRUE is used to ensure that this example will run withough error. The
# default value of 5 for nRepeats is sometimes not enought to find a starting
# design with a mixture problem.


dat<-gen.mixture(4,5)
desM<-optFederov(~(X1+X2+X3+X4+X5)^2-1,dat,15,nullify=TRUE)

# EXAMPLES 4
# Design augmenation can be obtained by setting augment=TRUE, and placing the row numbers
# of the design to be agmented in rows. Augmentation is often used to (1) add a new variable
# to an existing design or (2) to increase the complexity of the model. The following illustrates
# adding a variable to an existing design using desD above. It is assumed that all runs of the
# existing design have been made at the -1 level of the new variable:

dat<-gen.factorial(levels=3,nVars=3,varNames=c("A","B","C"))
desA<-optFederov(~quad(.),dat,nTrials=25,augment=TRUE,rows=desD$rows)

# The half fraction in desH, can be augmented to support an additional term:

dat<-gen.factorial(levels=2,nVars=4,varNames=c("A","B","C","D"))
desH<-optFederov(~.,dat,8)
desH2<-optFederov(~A+B+C+D+I(A*B),dat,10,aug=TRUE,rows=desH$rows)

# EXAMPLES 5
# Optimal approximate theory designs have non-zero probabilities only on support points.
# For the first example above the approximate theory design is as follows. It shows
# that all points in the cubic lattice are support points. The D for this 
# design is 0.474 which may be compared with the D of 0.463 of the first example to
# indicate that that exact design had a D-efficiency of 97\%. The lower bound Dea
# was 82\%.

dat<-gen.factorial(levels=3,nVars=3,varNames=c("A","B","C")) 
desDA<-optFederov(~quad(A,B,C),dat,eval=TRUE,approx=TRUE)

# The largest proportions will be rounded if nTrials is specified.

desDAN<-optFederov(~quad(A,B,C),dat,eval=TRUE,approx=TRUE,nTrials=15)
}

\author{
	Bob Wheeler \email{bwheeler@echip.com}
	
	Please cite this program as follows:

Wheeler, R.E. (2004). optFederov. \emph{AlgDesign}. The R project for statistical computing \url{http://www.r-project.org/}
}

}
\keyword{design}

\eof
\name{GVTable1}
\alias{GVTable1}
\title{Goos Vandebroek Table 1}
\usage{
	data(GVTable1)
}

\description{
	A blocked experimental design from Goos and Vandebroek (2003).

	A matrix giving one whole plot and four within plot variables, 
		in 21 blocks of 2.
	The same as TGTable5, but produced by Goos Vandoebroek algorithm.
}

\source{
	Goos, P. and Vandebroek, M. (2003). D-optimal split-plot designs with given numbers and 
		sizes of whole plots. \emph{Technometrics.} 45-3. 235-245.
}

\keyword{datasets}


\eof
\name{GVTable3}
\alias{GVTable3}
\title{Goos Vandebroek Table 3}
\usage{
	data(GVTable3)
}

\description{
	A blocked experimental design from Goos and Vandebroek (2003).

	A matrix giving one whole and two within plot variables, 9 blocks of 21.

}

\source{
	Goos, P. and Vandebroek, M. (2003). D-optimal split-plot designs with given numbers and 
		sizes of whole plots. \emph{Technometrics.} 45-3. 235-245.
}

\keyword{datasets}



\eof
\name{model.matrix.formula}
\alias{model.matrix.formula}
\title{Builds a model matrix}
\description{
 Produces a model matrix using expand.formula()
}
\usage{
model.matrix.formula(frml,data,...)
}

\arguments{
	\item{frml}{A formula.}
	\item{data}{A data.frame}
	\item{...}{Additional arguments passed to model.matrix.default()}
}
\details{
	This is a method function signaled by a formula as the first argument. 
	It causes the formula to be translated by expand.formula() before calling
      model.matrix.default().
}
\value{
	A matix.
}
\author{
	Bob Wheeler \email{bwheeler@echip.com}
	
	Please cite this program as follows:

Wheeler, R.E. (2004). model.matrix.formula. \emph{AlgDesign}. The R project for statistical computing \url{http://www.r-project.org/}
}

\keyword{design}


\eof
\name{optMonteCarlo}
\alias{optMonteCarlo}
\title{Optimal design via Monte Carlo}
\description{
  Finds a design using the specified criterion via Federov's algorithm
  applied to a random subset of all possible candidate points. 
}
\usage{
optMonteCarlo(frml,data,nTrials,approximate=FALSE,criterion="D",evaluateI=FALSE,
	space=NULL,mixtureSum=1,constraints=NULL,RandomStart=TRUE,nRepeats=5,nCand,
	nCandNull,DFrac=1,CFrac=1,args=FALSE)
}


\arguments{
	\item{frml}{Required: A formula starting with ~ which will be used with 
		model.matrix() to create a model matrix. If there are mixture variables,
		the constant term is suppressed.}
	\item{data}{Required: A data frame with 7 or 8 columns. See details below
		for specifics} 
	\item{nTrials}{number trials in design -- must be greater than the
	  number of terms in the model, if  missing will be set to the
	  number of model terms in the model  plus five.}
	\item{approximate}{When FALSE, an exact design in nTrails will be calculated. When TRUE the proportions for
	 an approximate theory design will be calculated. If nTrials is set, any proportion less than 1/(2*maxIteration)
	 will be discarded before the proportions are efficiently rounded, otherwise all non-zero proportions will be 
	 shown: these are the support points.}
	\item{criterion}{"D", "A", or "I"}
	\item{evaluateI}{TRUE if I is to be evaluated in addition to the
	  other criteria -- slower because of calculations for I}
	\item{space}{If the criterion is "I" or evaluate I is true, the space over which the I criterion is to
	  be evaluated may be input. It should be a matrix with the same column types and names as in data. If
	  space is not input the evaluation will be done over the space described by data.}	
	\item{constraints}{A function taking a vector argument with length
	  equal to the number of variables, and returning TRUE if the vector is 
	  inside the constrained region}
	\item{mixtureSum}{The mixture variables, if any, will sum to this value.}
	\item{RandomStart}{When TRUE, the starting design will be chosen
	  at random, otherwise nullification will be used. Note: the nullifcation
	  used here is different and much slower than that in optFederov().}
	\item{nRepeats}{number of times to retry the entire process}
	\item{nCand}{number of candidate points to generate, if missing,
	  it will be 10 times the number of terms}
	\item{nCandNull}{Number of candidate points to use for
	  nullification. If missing it will be set to nCand}
	\item{DFrac}{Fraction of design used in search: 1 uses all of
	  them, 0 only the one with the smallest variance}
	\item{CFrac}{Fraction of candidate set searched : 1 uses all of
	  them, 0 only the one with the largest variance}
	\item{args}{If TRUE, the actual arguments to the function including the starting 
		random number seed will be output.}
}
\details{
  The columns of the input data frame are as follows. The columns need
	not be named. It is probably best to avoid naming the variables with
	single letters, especially "I" -- use paste(), as in the examples. For each variable
	nLevels are randomly generated between low and high, inclusive, and then rounded
	with round. For integer levels, round should be set to 0.

	\describe{
	\item{var:}{The names of the variables.}
	\item{low:}{The lower limit of the range for each variable. Ignored for mixtures.} 
	\item{high:}{The upper limit of the range for each variable. Ignored for mixtures.} 
	\item{center:}{The centering value for each variable. Ignored for mixtures.}
	\item{nLevels:}{The number of levels for each variable. Ignored for mixture variables.}
	\item{round:}{The number of decimal digits for the levels. The levels are randomly and 
		uniformly chosen between low and high, and this parameter controls the number of 
		trailing digits. The max value for mixture variables in this vector is used to 
		round all mixture variables.}
	\item{factor:}{TRUE, FALSE depending on whether or not the variable is a factor. Note: other
		columns will be reset to conform to a nLevels factor.}
	\item{mix:}{TRUE if the variable is a mixture variable. This column may
		be omitted if there are no mixture variables.}
	}

  Candidate lists required by \code{optFederov()} increase with the number of
  variables, and can easily exceed storage capacity and can require
  excessive amounts of time to process. To overcome this problem,
  \code{optMonteCarlo()}, generates at random  \code{nCand}  points from a putative 
  candidate list.

  For non-mixture variables, \code{optMonteCarlo()} samples from the putative
  candidate list by choosing random levels inside the limits given by
  \code{low} and \code{high} in \code{data}. These are rounded to the 
  number of levels given by \code{nLevels} in \code{data} and to the 
  number of decimal digits given by \code{round} in \code{data}.

  For mixture variables, \code{optMonteCarlo()} samples from the putative
  candidate list by choosing random levels between 0 and 1, rounded to
  the maximum in the \code{round} column of \code{data}, and such that the sum over 
  all variables is equal to \code{mixtureSum}.
  
  If a constraint function is supplied in \code{Constraints}, it is
  applied, and results which do not meet the constraint are
  discarded. The constraint function should be written to process 
  uncentered variables.

  The above procedures are repeated until \code{nCand} candidate points
  are found.

  Nullification, successively adds points to a design until n points are 
  found. This is the same procedure that is in \code{optFederov} except
  that each new point is selected from a new sampling of the putative
  candidate points. In general, this will produce better designs that
  those from a random start.

  The entire process is repeated \code{nRepeats} times, and the best
  result is reported. The methodology compares favorably with an
  exhaustive search where the entire candidate list is searched by
  \code{optFederov()}.
  
  The random numbers used in these calculations are controlled by the
  usual R random number mechanism.

}

\note{

Details of the calculations and expanded examples may be found in ``Comments on algorithmic design'' in the
doc subfolder of this project.
}

\value{
The output is the same list as from \code{optFederov}, but the criteria
values are relative to the randomly chosen subsets of the putative candidate 
space. In general, they should not differ greatly from those obtained by an 
exhaustive search.

  \item{D}{The kth root of the generalized variance: \eqn{det(M)^{1/k}}{det(M)^(1/k)}, where \eqn{det(M)} is the
    determinant of the normalized dispersion matrix \eqn{M} --
    i.e. \eqn{M=Z'Z/n}, where \eqn{Z=X[rows,]}}
  \item{A}{The average coefficient variance: \eqn{trace(Mi)/k}, where \eqn{Mi} is the inverse of \eqn{M}.}
  \item{I}{The average prediction variance over X, which can be shown to be \eqn{trace((X'X*Mi)/N)}. 
	This is calculated only when I is the criterion or when \code{evaluateI} is TRUE.}
  \item{Ge}{The minimax normalized variance over X, expressed as an efficiency with respect to the optimal approximate
	theory design. It is defined as \eqn{k/max(d)}, where \eqn{max(d)} is the maximum normalized
    	variance over \eqn{X} -- i.e. the max of \eqn{x'(Mi)x}, over all rows \eqn{x'}
    	of \eqn{X}.}
  \item{Dea}{A lower bound on \code{D} efficiency for approximate theory designs. It is equal to \eqn{exp(1-1/Ge)}.}
  \item{Design}{The design.} 
  \item{args}{A list of the actual arguments used in this call.}

}
\author{
	Bob Wheeler \email{bwheeler@echip.com}
	
	Please cite this program as follows:

Wheeler, R.E. (2004). optMonteCarlo. \emph{AlgDesign}. The R project for statistical computing \url{http://www.r-project.org/}
}

\examples{

# EXAMPLE 1
# The data.frame in data might look like the following:
data<-data.frame(var=paste("X",1:6,sep=""),low=c(1,1,1,0,0,0),
high=c(3,3,3,1,1,1),center=c(2,2,2,0,0,0),nLevels=3,
round=1,factor=0,mix=c(FALSE,FALSE,FALSE,TRUE,TRUE,TRUE))
data

# and the design:

optMonteCarlo(~(X1+X2+X3)^2+X4+X5+X6,data)

# Example 2
# Standard designs will often be produced, just as 
# they will with optFederov(). For example,
# a half fraction of a 2^4:
data<-data.frame(paste("X",1:4,sep=""),-1,1,0,2,0,0)
data
optMonteCarlo(~.,data,nTrials=8)

# Example 3
# optMonteCarlo() can treat much larger problems than can 
# optFederov().  For example, optFederov()
# requires a candidate list of 3^20 points for
# a 20 variable, 3 level candidate list -- about
# 25 gigabytes. If the model is quadratic, this must
# be multiplied by about 12. There are other storage
# requirements internal to optFederov() which easily
# double this value. optMonteCarlo() since it only samples
# from the putative candidate list, has no difficulty 
# with a problem of this size. The criterion values
# appearing in the output of optMonteCarlo() are based on
# these samples, but their values seem to be reasonable
# correct, as the following shows: (These are commented
# out for those who have a slow machine.)

dat<-gen.factorial(levels=3,nVar=8)
#desF<-optFederov(~quad(.),dat,eval=TRUE)
#desF[1:5]

data<-data.frame(paste("X",1:8,sep=""),-1,1,0,3,0,0)
#desH<-optMonteCarlo(~quad(.),data,Rand=FALSE,eval=TRUE)
#desH[1:5]

# The following is a 20 variable quadratic. Uncomment
# and wait a while, even if you have a fast machine.
# Note: nRepeats has been changed from its default.
# Note: criterion values for exact designs are often
# far from approximate theory optima; hence, Ge and De
# will be small.

data<-data.frame(paste("X",1:20,sep=""),-1,1,0,3,0,0)
#desBig<-optMonteCarlo(~quad(.),data,nRepeats=1)

# The following will produce improved criterion values

#desNBig<-optMonteCarlo(~quad(.),data,Rand=FALSE,nRepeats=1)

# EXAMPLE 4
# Practically infeasible combinations of variable are 
# common. Designs may be produced which avoid such
# combinations by using a constraint function. Suppose,
# for example that one corner of a cubic box is not
# feasible, then the following will produce a design
# that makes no use of this corner.

Constraints<-function(x){!(x[1]>0.75 && x[2]>0.75)}
data<-data.frame(paste("X",1:4,sep=""),-1,1,0,3,0,0)
desC<-optMonteCarlo(~.,data,con=Constraints)

# The above just removes a corner. Increasing the
# number of levels will remove points along the
# boundary.

data<-data.frame(paste("X",1:4,sep=""),-1,1,0,11,3,0)
desC2<-optMonteCarlo(~.,data,con=Constraints)

}
\keyword{design}

\eof
\name{TGTable3}
\alias{TGTable3}
\title{Trinca Gilmour Table 3}
\usage{
	data(TGTable3)
}

\description{
	A blocked experimental design from Trinca and Gilmour (2001).

	A data.frame giving four three level variable in five blocks. There
		are 45 trials in all.
}

\source{
	Trinca, L.A. and Gilmour, S.G. (2001). Multistratum response surface designs. 
		{\emph Technometrics}. 43-1. 25-33.
}

\keyword{datasets}


\eof
\name{TGTable5}
\alias{TGTable5}
\title{Trinca Gilmour Table 5}
\usage{
	data(TGTable5)
}

\description{
	A blocked experimental design from Trinca and Gilmour (2001).

	A matrix giving one whole plot and four within plot variables, 
		in 21 blocks of 2.
}

\source{
	Trinca, L.A. and Gilmour, S.G. (2001). Multistratum response surface designs. 
		{\emph Technometrics}. 43-1. 25-33.
}

\keyword{datasets}


\eof
\name{optBlock}
\alias{optBlock}
\title{Optimal design blocking}

\usage{

optBlock(frml,withinData,blocksizes,rows=NULL,wholeBlockData=NULL,center=FALSE,
	nRepeats=5,criterion="D",args=FALSE)

}
\arguments{
	\item{frml}{This may be omitted if \code{data} is the fully model expanded candidate list.
	 If present it should be a formula starting with \code{~} which describes the model using
	 variable names from \code{data}. It may be \code{~}. if all variables from \code{data} are to be 
	 used linearly. It may also be \eqn{~.^p}, where p is an integer. In addition to the usual 
	 operators, quad(), cubic() and cubicS() may be used to expand variables from \code{data} 
	 into polynomial models.}
	\item{withinData}{A matrix or data.frame describing the variables. If the columns are not 
       named, they will be assumed to have the names X1,X2, etc. Although data may be input as
	 global variables used in frml, it is preferable to input it here. The number of rows in \code{withinData}
	 must be at least as large as the sum of the number of terms plus the number of blocks.}
	\item{blocksizes}{A vector giving the block sizes for each block. The length of \code{blocksizes}
		specifies the number of blocks.}
	\item{rows}{Row numbers (not rownames) of withinData rows to be used as a starting blocked design.}
	\item{wholeBlockData}{A matrix or data.frame describing the whole block variables. The number of
		rows should equal the length of blocksizes. Each row should
		correspond to the settings of the whole block variable in a block.}
	\item{center}{If TRUE, the withinData and wholeBlockData will be centered.}
	\item{nRepeats}{The number of times the entire process is repeated.}
	\item{criterion}{"D":D-criterion: "OB","OBS": orthogonal blocks, unscaled and scaled; "Dp","Dpc": 
		Product of block determinants, uncentered and centered.}
	\item{args}{If TRUE, the actual arguments to the function including the starting random number
		 seed should be output.}
      }

\description{Blocking of experimental designs using various criteria.}
      
\details{
Cook and Nachtsheim (1989) developed a blocking algorithm using the D criterion, which not only finds blocks, but attempts to optimize the design itself. The package Dopt, converted by V.N. Venables, implements this technology -- unfortunately there seem to be bugs in the FORTRAN code. 

This function uses various criteria to block either pre-existing designs or to creates a new blocked design from a candidate list. There may be interactions between whole plot and within plot factors, and it may be used to produce multistratum blocked designs. 

The algebraic expressions used here for the D criterion are quite different than those used by either Cook and Nachtshim or Goos and Vandebroek. They are described in \emph{Comments on algorithmic design}, a paper accompanying this package.

Although the D criterion produces good designs for all blocks, the allocation for individual blocks can often be improved upon by the use of either the Dp or Dpc criterion, which optimize the product of the determinants of the individual blocks. The Dp criterion uses uncentered blocks, the Dpc uses centered blocks. 

Blocking may also be done using the orthogonal blocking procedure of Nguyen. This comes in two flavors OB and OBS which differ in that for OBS, the columns of S are scaled by division with the sample variance of the data, thus deemphasizing squared and other large terms.

Blocking may be done when whole plot factors interact with within plot factors. Split plot experiments are sometimes of this nature. Goos and Vandebroek (2003) developed a procedure for this problem which algorithmically blocks using a candidate list and the ratio between the whole and within variances. Trinca and Gilmour (2001) give an algorithm which does not depend on the ratio of the variances. The present procedure assumes that the whole blocks and their factors, have been decided upon in advance. It makes no assumptions about the ratio between whole and within variances. 
}

\note{
Details of the calculations and expanded examples may be found in ``Comments on algorithmic design'' in the
doc subfolder of this project.

}

\references{
Atkinson, A.C. and Donev, A.N. (1989). The construction of exact D-optimum experimental designs with application to blocking response surface designs. {\emph Biometrika}. 76. 515-526.

Cook, R.D. and Nachtsheim, C.J. (1989). Computer-aided blocking of factorial and response-surface designs. \emph{Technometrics.} 31-3. 339-346.

Goos, P. and Vandebroek, M. (2003). D-optimal split-plot designs with given numbers and sizes of whole plots. \emph{Technometrics.} 45-3. 235-245.

Nguyen, Nam-Ky. (2001). Cutting experimental designs into blocks. {\emph AusNZJSt}. 43-3. 367-374.

Trinca, L.A. and Gilmour, S.G. (2000). An algorithm for arranging response surface designs in small blocks. {\emph Computational Statistics and Data Analysis}. 33. 25-43.

Trinca, L.A. and Gilmour, S.G. (2001). Multistratum response surface designs. {\emph Technometrics}. 43-1. 25-33.

}

\examples{
# Blocking the design for a quadratic polynomial in three variables into two 
# seven trial blocks:

dat<-gen.factorial(3,3,varNames=c("A","B","C"))
desD<-optFederov(~quad(.),dat,nTrials=14,eval=TRUE) # Choose an optimum 14 trail design.
optBlock(~quad(.),desD$design,c(7,7))

# Letting optBlock() search the dat candidate list instead of first choosing a 
# 14 trial design.
optBlock(~quad(.),dat,c(7,7))


# A block design for 7 treatments in 7 blocks of size 3. Note how withinData 
# is recycled to fill out the blocksize requirements.

BIB<-optBlock(~.,withinData=factor(1:7),blocksizes=rep(3,7))

# This is a balanced incomplete block design as may be seen from:

crossprod(table(BIB$rows,c(rep(1:7, rep(3,7)))))

# A partially balanced incomplete block design with two associate classes:

tr<-factor(1:9)
PBIB<-optBlock(~.,withinData=tr,blocksizes=rep(3,9))

crossprod(table(PBIB$rows,c(rep(1:9, rep(3,9)))))


# Two fractions of a 2^(4-1).

dat<-gen.factorial(2,4)
od<-optBlock(~.,dat,c(8,8)) 

# The blocks are not themselves orthogonal even though the entire design is optimal.

bk<-data.matrix(od$Blocks$B1)
t(bk)\%*\%bk 

# Better blocks may be obtained as follows, but note that they are not generally 
# the fractions that would be obtained by confounding the third order interaction.

od<-optBlock(~.,dat,c(8,8),criterion="Dpc",nR=10)
bk<-data.matrix(od$Blocks$B1)
t(bk)\%*\%bk

# Blocking with whole plot factors. Note that the 27 rows of within are recycled 
# to make the 54 trial blocked design. 

within<-expand.grid(A=c(-1,0,1),B=c(-1,0,1),C=c(-1,0,1))
whole<-expand.grid(D=factor(1:3),E=factor(1:3))
od<-optBlock(~D+E*(quad(A,B,C)),withinData=within,blocksizes=rep(6,9),wholeBlockData=whole)

# Either withinData, or wholeBlockData may be an approximate theory optimial design 
# produced by optFederov() for nTrials. The first column in the optFederov() output 
# design, named "Rep..", is used to replicate the trials. 

within<-optFederov(~quad(A,B,C),within,nT=54,approx=TRUE)
od<-optBlock(~D+E*(quad(A,B,C)),withinData=within$design,blocksizes=rep(6,9),wholeBlockData=whole)

}


\value{

  \item{D}{\eqn{det(M)^{1/k}}{det(M)^(1/k)}, where \eqn{det(M)} is the
    determinant of the normalized dispersion matrix \eqn{M} --
    i.e. \eqn{M=X'X/N}, where each row of X has had the appropriate block mean subtracted.}
  \item{Dp (Dpc)}{\eqn{\left(\prod_1^b (det(M_i))^{1/k}\right)^{1/b}}{[prod (det(Mi)^(1/k)]^(1/b)}, 
	where \eqn{det(M_i)}{det(Mi)} is the determinant of the normalized dispersion matrix 
	for the uncentered (centered) block submatrix of \eqn{X}, and \eqn{b} is the number of blocks. The
	normalization is with respect to the number of observations in each block.}
  \item{diagonality or SS}{The diagonality is \eqn{(|M|/P)^{1/k}}, where P is the product of the diagonal
	elements of M. The SS is the sum of squares of S when the OB criterion is used.}
  \item{Blocks}{A list of the blocks, labeled B1, B2, etc.}
  \item{design}{A data.frame. The design with blocks stacked in order.}
  \item{rows}{Numeric row numbers of the design rows corresponding to the withinData rows.} 
  \item{args}{A list of the actual arguments used in this call.}

}

\author{
	Bob Wheeler \email{bwheeler@echip.com}

	Please cite this program as follows:

Wheeler, R.E. (2004). optBlock. \emph{AlgDesign}. The R project for statistical computing \url{http://www.r-project.org/}
}
\keyword{design}


\eof
\name{gen.factorial}
\alias{gen.factorial}
\title{Generates a full factorial design}
\description{
 A full factorial design is generated.
}
\usage{
	gen.factorial(levels, nVars=0, center=TRUE, factors="none",varNames=NULL)
}

\arguments{
	\item{levels}{A vector of levels for the variables. May be an integer if nVars is specified.}
	\item{nVars}{The number of variables.}
	\item{center}{If TRUE, all non-factors will be centered.}
	\item{factors}{If "all", all variables are factors, otherwise a vector of the variable numbers
		of the variables that are to be factors.}
	\item{varNames}{The names of the variables.}
}
\value{
	A factorial design in a data.frame.
}

\examples{
dat<-gen.factorial(3,3)
dat<-gen.factorial(c(3,2,3))
dat<-gen.factorial(3,3,factors="all")
dat<-gen.factorial(3,3,varNames=c("A","B","C"))

}

\author{
	Bob Wheeler \email{bwheeler@echip.com}
	
	Please cite this program as follows:

Wheeler, R.E. (2004). gen.factorial. \emph{AlgDesign}. The R project for statistical computing \url{http://www.r-project.org/}
}


\keyword{design}


\eof
\name{gen.mixture}
\alias{gen.mixture}
\title{Generate mixture}
\description{
 Creates a candidate list of mixture variables.
}
\usage{
gen.mixture(levels,vars)
}
\arguments{
	\item{levels}{An integer greater than 1. The number of levels of the mixture variables.}
	\item{vars}{Either the number of variables, or a character vector of variable names.}
}

\details{
Similar in function to gen.factorial(), but for mixture variables such that the rows sum to unity.
For \code{levels=2}, the identity matrix is produced, so that each variable is either 0 or 1. For 
\code{levels=3}, rows are added to the identity containing two variables at 1/2 and the others at 0. etc.

In general, a mixture model of degree d will have the same number of terms as the candidate
list generated by \code{gen.mixture} for \code{levels=d+1}, and this will be optimal. 

}

\author{
	Bob Wheeler \email{bwheeler@echip.com}
	
	Please cite this program as follows:

Wheeler, R.E. (2004). gen.mixture. \emph{AlgDesign}. The R project for statistical computing \url{http://www.r-project.org/}
}
\keyword{design}


\eof
